import datetime

from factory import SubFactory
from factory.django import DjangoModelFactory
from pytz import UTC

from django_features.system_message import models


class SystemMessageTypeFactory(DjangoModelFactory):
    class Meta:
        model = models.SystemMessageType

    name = "Info"
    icon = "information"


class SystemMessageFactory(DjangoModelFactory):
    class Meta:
        model = models.SystemMessage

    background_color = "#008DCC"
    begin = datetime.datetime(2025, 1, 1, tzinfo=UTC)
    text = "Hello World!"
    title = "System Info"
    type = SubFactory(SystemMessageTypeFactory)  # type: ignore



# Generated by Django 4.2.23 on 2026-02-24 11:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("custom_fields", "0006_add_filterable_to_customfield"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="customvalue",
            name="field",
        ),
        migrations.DeleteModel(
            name="CustomField",
        ),
        migrations.DeleteModel(
            name="CustomValue",
        ),
    ]



# Generated by Django 4.2.24 on 2025-10-30 12:07

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ("custom_fields", "0005_rename_text_to_label"),
    ]

    operations = [
        migrations.AddField(
            model_name="customfield",
            name="filterable",
            field=models.BooleanField(
                default=False, verbose_name="Als Filter anbieten"
            ),
        ),
    ]



from django.db import models
from django.utils.translation import gettext_lazy as _
from django_extensions.db.models import TimeStampedModel


class CustomValueQuerySet(models.QuerySet):
    def for_model(self, model: type[models.Model]) -> "CustomValueQuerySet":
        return self.select_related("field", "field__content_type").filter(
            field__content_type__app_label=model._meta.app_label,
            field__content_type__model=model._meta.model_name,
        )

    def for_type(self, model: type[models.Model]) -> "CustomValueQuerySet":
        return self.select_related("content_type").filter(
            field__type_content_type__app_label=model._meta.app_label,
            field__type_content_type__model=model._meta.model_name,
        )

    def default(self) -> "CustomValueQuerySet":
        return self.filter(field__type_id__isnull=True)

    def default_for(self, model: type[models.Model]) -> "CustomValueQuerySet":
        return self.for_model(model).default()


class AbstractBaseCustomValue(TimeStampedModel):
    order = models.PositiveIntegerField(_("Reihenfolge"), default=0)
    label = models.CharField(verbose_name=_("Label"), null=True, blank=True)
    value = models.JSONField(verbose_name=_("Wert"), null=True, blank=True)

    objects = CustomValueQuerySet.as_manager()

    class Meta:
        abstract = True

    def __str__(self) -> str:
        return self.text or ""

    @property
    def text(self) -> str:
        return self.label or str(self.value)
